#include "serialization.h"

#include <list>
#include <map>
#include <iostream>
#include <string>
#include <sstream>
#include <fstream>

#include "unittest.h"

using std::cout;
using std::cerr;
using std::endl;
using std::list;
using std::map;
using std::string;
using std::stringstream;
using std::istream;
using std::ostream;
using std::ifstream;
using std::ofstream;
using std::pair;

//only can explicitly serialize data when the type is built-in
template <>
void SERIALIZER::Serialize<int>(string name, int & t)
{
	_GENERATEUID_(&t,uid);
	unsigned long long fileuid = this->FormatInfo(name, START, uid, true);
	if (!this->IsOutput())
		pointer_resolution_map[fileuid] = &t;
	this->SerializeData(t);
	this->FormatInfo(name, END, uid, true);
}
template <>
void SERIALIZER::Serialize<bool>(string name, bool & t)
{
	_GENERATEUID_(&t,uid);
	unsigned long long fileuid = this->FormatInfo(name, START, uid, true);
	if (!this->IsOutput())
		pointer_resolution_map[fileuid] = &t;

	//utilize integers as our writing format
	int temp(0);
	if (this->IsOutput())
	{
		if (t)
			temp = 1;
		else
			temp = 0;
	}
	this->SerializeData(temp);
	if (!this->IsOutput())
	{
		if (temp == 0)
			t = false;
		else
			t = true;
	}

	this->FormatInfo(name, END, uid, true);
}
template <>
void SERIALIZER::Serialize<float>(string name, float & t)
{
	_GENERATEUID_(&t,uid);
	unsigned long long fileuid = this->FormatInfo(name, START, uid, true);
	if (!this->IsOutput())
		pointer_resolution_map[fileuid] = &t;
	this->SerializeData(t);
	this->FormatInfo(name, END, uid, true);
}
template <>
void SERIALIZER::Serialize<double>(string name, double & t)
{
	_GENERATEUID_(&t,uid);
	unsigned long long fileuid = this->FormatInfo(name, START, uid, true);
	if (!this->IsOutput())
		pointer_resolution_map[fileuid] = &t;
	this->SerializeData(t);
	this->FormatInfo(name, END, uid, true);
}
template <>
void SERIALIZER::Serialize<string>(string name, string & t)
{
	_GENERATEUID_(&t,uid);
	unsigned long long fileuid = this->FormatInfo(name, START, uid, true);
	if (!this->IsOutput())
		pointer_resolution_map[fileuid] = &t;
	this->SerializeData(t);
	this->FormatInfo(name, END, uid, true);
}

ostream& operator <<(ostream &os,const TEST_FRAMEWORK::TEST_VERTEX &v)
{
	os << v.x << "," << v.y << "," << v.z;
	return os;
}

QT_TEST(serialization_test_xml)
{
	stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_XML_OUTPUT serialize_output(teststream);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		SERIALIZER_XML_INPUT serialize_input(teststream);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");
	QT_CHECK_EQUAL(player2.curpos,&player2.position);
	TEST_FRAMEWORK::TEST_VERTEX * nullptr(NULL);
	QT_CHECK_EQUAL(player2.nextpos,nullptr);

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);

			if (count == 2)
				QT_CHECK_EQUAL(player2.lastpos,&(*i));
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);
}

QT_TEST(serialization_test_xml_no_ptrs)
{
	stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_XML_OUTPUT serialize_output(teststream);
		serialize_output.SetPointerTracking(false);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		SERIALIZER_XML_INPUT serialize_input(teststream);
		serialize_input.SetPointerTracking(false);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);

	//ofstream ofile("test.xml");
	//ofile << teststream.str() << endl;
}

QT_TEST(serialization_test_bin)
{
	stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_BIN_OUTPUT serialize_output(teststream);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		SERIALIZER_BIN_INPUT serialize_input(teststream);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");
	QT_CHECK_EQUAL(player2.curpos,&player2.position);
	TEST_FRAMEWORK::TEST_VERTEX * nullptr(NULL);
	QT_CHECK_EQUAL(player2.nextpos,nullptr);

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);

			if (count == 2)
				QT_CHECK_EQUAL(player2.lastpos,&(*i));
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);

	//ofstream binfile("test.bin",ofstream::binary);binfile.write(teststream.str().c_str(),teststream.str().length());cout << "Outputting test.bin" << endl;
}

//this format is used often for network traffic, so we explicitly check it
QT_TEST(serialization_test_bin_no_ptrs)
{
	stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_BIN_OUTPUT serialize_output(teststream);
		serialize_output.SetPointerTracking(false);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		SERIALIZER_BIN_INPUT serialize_input(teststream);
		serialize_input.SetPointerTracking(false);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);

	//ofstream binfile("test-noptrs.bin",ofstream::binary);binfile.write(teststream.str().c_str(),teststream.str().length());
}

QT_TEST(serialization_test_configfile)
{
    stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_CONFIGFILE_OUTPUT serialize_output(teststream);
		serialize_output.SetPointerTracking(true);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//ofstream ofile("test.txt");
	//ofile << teststream.str() << endl;

	//read input
	{
		SERIALIZER_CONFIGFILE_INPUT serialize_input(teststream);
		serialize_input.SetPointerTracking(true);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");
	QT_CHECK_EQUAL(player2.curpos,&player2.position);
	TEST_FRAMEWORK::TEST_VERTEX * nullptr(NULL);
	QT_CHECK_EQUAL(player2.nextpos,nullptr);

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
             count = 0;
			 i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);

			if (count == 2)
				QT_CHECK_EQUAL(player2.lastpos,&(*i));
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);
}

QT_TEST(serialization_test_configfile_no_ptrs)
{
	stringstream teststream; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_CONFIGFILE_OUTPUT serialize_output(teststream);
		serialize_output.SetPointerTracking(false);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//ofstream ofile("test.txt");
	//ofile << teststream.str() << endl;

	//read input
	{
		SERIALIZER_CONFIGFILE_INPUT serialize_input(teststream);
		serialize_input.SetPointerTracking(false);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);

	/*for (std::map <string, TEST_FRAMEWORK::TEST_VERTEX>::iterator i = player2.mymap.begin(); i != player2.mymap.end(); i++)
	{
	    cout << i->first << endl;
	}*/

	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);
}

ostream& operator <<(ostream &os,const SERIALIZER_TREE_VALUE &v)
{
	os << v.value << " (" << v.uid << ")";
	return os;
}

QT_TEST(serialization_test_tree)
{
	SIMPLETREE <string, SERIALIZER_TREE_VALUE> tree; //holds input/output

	//write output
	{
		TEST_FRAMEWORK::TEST_PLAYER player1(true); //our populated player
		SERIALIZER_TREE_OUTPUT serialize_output(tree);
		_STARTSERIALIZE_((&serialize_output),player1);
	}//player1 goes out of scope

	//tree.DebugPrint();

	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		SERIALIZER_TREE_INPUT serialize_input(tree);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");
	QT_CHECK_EQUAL(player2.curpos,&player2.position);
	TEST_FRAMEWORK::TEST_VERTEX * nullptr(NULL);
	QT_CHECK_EQUAL(player2.nextpos,nullptr);

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);

			if (count == 2)
				QT_CHECK_EQUAL(player2.lastpos,&(*i));
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);
}

//check for compatibility of the binary format
QT_TEST(serialization_test_bin_compatibility)
{
	TEST_FRAMEWORK::TEST_PLAYER player2(false); //our blank slate

	//read input
	{
		ifstream teststream(string("data/test/test.bin").c_str(),ifstream::binary);
		QT_CHECK(teststream);
		SERIALIZER_BIN_INPUT serialize_input(teststream);
		serialize_input.StartSerialize("player1", player2);
	}

	//do our checks
	QT_CHECK_EQUAL(player2.position,TEST_FRAMEWORK::TEST_VERTEX(1,1,1));
	QT_CHECK_EQUAL(player2.animframe,1337);
	QT_CHECK_EQUAL(player2.name,"Test player");
	QT_CHECK_EQUAL(player2.curpos,&player2.position);
	TEST_FRAMEWORK::TEST_VERTEX * nullptr(NULL);
	QT_CHECK_EQUAL(player2.nextpos,nullptr);

	//create comparison values
	list <int> local_simplelist;
	local_simplelist.push_back(9);
	local_simplelist.push_back(8);
	list <TEST_FRAMEWORK::TEST_VERTEX> local_complexlist;
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(7,6,5));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(4,3,2));
	local_complexlist.push_back(TEST_FRAMEWORK::TEST_VERTEX(1,0,-1));

	QT_CHECK_EQUAL(player2.simplelist.size(),local_simplelist.size());

	{
		list <int>::iterator i;
		list <int>::iterator n;
		for (i = player2.simplelist.begin(),
		     n = local_simplelist.begin();
		     i != player2.simplelist.end() && n != local_simplelist.end(); i++,n++)
		{
			QT_CHECK_EQUAL(*i,*n);
		}
	}

	QT_CHECK_EQUAL(player2.complexlist.size(),local_complexlist.size());

	{
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator i;
		list <TEST_FRAMEWORK::TEST_VERTEX>::iterator n;
		int count;
		for (i = player2.complexlist.begin(),
		     n = local_complexlist.begin(),
				     count = 0;
				     i != player2.complexlist.end() && n != local_complexlist.end(); i++,n++,count++)
		{
			QT_CHECK_EQUAL(*i,*n);

			if (count == 2)
				QT_CHECK_EQUAL(player2.lastpos,&(*i));
		}
	}

	QT_CHECK_EQUAL(player2.mymap.size(), 2);
	QT_CHECK_EQUAL(player2.mymap["testing123"], TEST_FRAMEWORK::TEST_VERTEX(1,2,3));
	QT_CHECK_EQUAL(player2.mymap["testing654"], TEST_FRAMEWORK::TEST_VERTEX(6,5,4));

	QT_CHECK_EQUAL(player2.myvector.size(), 2);
	QT_CHECK_EQUAL(player2.myvector[0], TEST_FRAMEWORK::TEST_VERTEX(5,4,3));
	QT_CHECK_EQUAL(player2.myvector[1], TEST_FRAMEWORK::TEST_VERTEX(2,1,0));

	QT_CHECK_EQUAL(player2.player_description, "Hello there.\nHow are you??");

	QT_CHECK_EQUAL(player2.alive, true);
	QT_CHECK_EQUAL(player2.aiming, false);

	QT_CHECK_EQUAL(player2.curtime, 0.123456789);
}
